// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: Action.java
// Classes: Action
// Original Author: jrobbins@ics.uci.edu
// $Id: Action.java,v 1.8 1997/01/16 23:08:03 jrobbins Exp $

package uci.graphedit;

import java.util.*;



/**
 * Abstract class for all editor actions. The editor serves as a
 * command shell for executing actions in much the same way that a
 * DOS or UNIX commmand command shell executes programs. Each action
 * can have a Hashtable of "command-line" arguments and also look at
 * global variables (its environment). Once an instance of an Action
 * is made, it can be sent the doIt() and undoIt() messages to perform
 * that action. <p>
 * needs-more-work: canDoIt, canUndoIt predicates control graying. <p>
 * needs-more-work: Editor will keep a history of recent actions for
 * undo. <p>
 * @see Editor
 * @see ExecuteActionWindow
 */
public abstract class Action {

  /** Arguments that configure the Action instance */
  protected Properties _args;

  /** Construct a new Action with the given arguments */
  public Action(Properties args) { _args = args; }

  /** Construct a new Action with no arguments */
  public Action() { this(new Properties()); }

  /** Return a name for this action suitable for display to the user */
  public String name() { return this.getClass().getName(); }

  /** Return a URL that has user and programmer documentation */
  public String about() {
    return "http://www.ics.uci.edu/~jrobbins/gef/Docs.html#" +
		 getClass().getName();
  }

  /** Perform whatever action this Action is meant to do. Subclasses
   * should override this to do whatever is intended. When the action
   * executes, it should store enough information to undo itself later
   * if needed.
   * @param e  The event that caused this Action to be
   * performed. Generally it is a bad idea to depend on the
   * contents of this event too much.
   */
  public abstract void doIt(java.awt.Event e);

  /** Undo the action using information stored during its
   * execution. <p>
   * needs-more-work: This is not currently implemented.
   */
  public abstract void undoIt();

  // needs-more-work: do I need a separate redo()? Should doIt() take
  // flag to indicate if this is the first time the action is being
  // done, or it it is actually being redone? What information does
  // undoIt() need to store to support redo?

  /** A list of Action instances that should appear in lists for the user
   to pick from */
  private static Vector _registeredActions = new Vector();

  /** Return a list of "well-known" Action instances that should
   * appear in lists * for the user to pick from. 
   * @see ExecuteActionWindow */
  public static Enumeration registeredActions() {
    return _registeredActions.elements();
  }

  /** Add a "well-known" Action */
  public static void register(Action a) { _registeredActions.addElement(a); }

  /** Return the "well-known" action at a given index. Useful for
    displaying a list of "well-known" actions. */
  public static Action actionAtIndex(int i) {
    return (Action) _registeredActions.elementAt(i);
  }
  
} /* end class Action */
